package applicative

import org.scalatest._
import testutil.PendingIfUnimplemented

class ApplicativeSpec extends FlatSpec with Matchers with AppendedClues with PendingIfUnimplemented {
  implicit val optionApplicative = new Applicative[Option] {
    override def unit[A](a: A): Option[A] = Some(a)
    override def ap[A, B](ff: Option[A => B])(fa: Option[A]): Option[B] = ff.flatMap(f => fa.map(f))
  }
  implicit val listApplicative = new Applicative[List] {
    override def unit[A](a: A): List[A] = List(a)
    override def ap[A, B](ff: List[A => B])(fa: List[A]): List[B] = ff.flatMap(f => fa.map(f))
  }

  "map" should "work the same when implemented via ap" in {
    val f: Int => Int = i => i * 3
    val g: String => String = s => "Hello " + s
    val h: Int => String = _.toHexString

    val someint = Some(2)
    val somestring = Some("world")

    optionApplicative.map(someint)(f) shouldBe someint.map(f)
    optionApplicative.map(somestring)(g) shouldBe somestring.map(g)
    optionApplicative.map(None)(g) shouldBe None.map(g)
    optionApplicative.map(None)(g) shouldBe None.map(g)
  }

  "map2" should "combine two monadic values" in {
    optionApplicative.map2(Some(1), Some(2))((a, b) => a + b) shouldBe Some(3)
    optionApplicative.map2(Option.empty[Int], Some(2))((a, b) => a + b) shouldBe None
    optionApplicative.map2(Some(2), Option.empty[Int])((a, b) => a + b) shouldBe None
  }

  "compose" should "create nested element with unit" in {
    val ola: Applicative[Lambda[a => Option[List[a]]]] = optionApplicative.compose(listApplicative)

    ola.unit(5) shouldBe Some(List(5))
  }

  it should "implement map2 or ap correctly" in {
    val ola: Applicative[Lambda[a => Option[List[a]]]] = optionApplicative.compose(listApplicative)

    val optList = Some(List(1, 2, 3, 4))

    ola.map(optList)(_ + 10) shouldBe Some(List(11, 12, 13, 14))
  }
}
