package monads

import org.scalatest._
import testutil.PendingIfUnimplemented

class MonadSpec extends FlatSpec with Matchers with AppendedClues with PendingIfUnimplemented {
  "map2" should "combine two monadic values" in {
    tupleMonad.map2(Tuple1(1), Tuple1(2))((a, b) => a + b) shouldBe Tuple1(3)
  }

  "sequence" should "correctly sequence all values" in {
    tupleMonad.sequence(List(
      Tuple1(1),
      Tuple1(2),
      Tuple1(3)
    )) shouldBe Tuple1(List(1,2,3))
  }

  "compose" should "correctly compose monad functions" in {
    val f: Int => Tuple1[Double] = a => Tuple1(a/2.0)
    val g: Double => Tuple1[String] = a => Tuple1(a.toString)

    tupleMonad.compose(f)(g)(3) shouldBe Tuple1("1.5")
  }

  val tupleMonad = new Monad[Tuple1] {
    def unit[A](a: A): Tuple1[A] = Tuple1(a)
    def flatMap[A, B](fa: Tuple1[A])(f: A => Tuple1[B]): Tuple1[B] =
      f(fa._1)
  }
}
